
// ignore_for_file: unnecessary_import, prefer_const_declarations, unnecessary_this, unused_import, prefer_final_fields

import 'dart:async';
import 'dart:convert';
import 'dart:core';

import 'package:flutter/services.dart';
import 'package:fgx_super_pay/model/entity/pay_package_entity.dart';
import 'package:fgx_super_pay/model/pay_resouce.dart';
import 'package:fgx_super_pay/pay_plugin.dart';
import 'package:flutter/cupertino.dart';
import 'package:package_info/package_info.dart';

typedef OnLogEvent = void Function(String eventName, Map<String, dynamic> params);

class FgxSuperPay {
  FgxSuperPay._internal() {
    _payResource = PayResource();
  }
  static FgxSuperPay _singleton = FgxSuperPay._internal();
  factory FgxSuperPay() => _singleton;

  late String _pureBasicUrl;
  late String _preferPackage;
  late String _localPackage;

  late OnLogEvent? _onLogEvent;
  late PayResource _payResource;
  late PayPackagePayPackageList? _selectPackage;
  late PayPackageEntity _payPackage;

  static final priceKey = "pcccc";
  static final periodKey = "poooo";
  static final periodTypeKey = "ppppp";
  static final pureBasicUrlKey = "pureBasicUrl";
  static final packageNameKey = "packageName";
  static final serviceNameKey = "serviceName";

  void init({
    required String packageUrl,
    required String pureBasicUrl,
    required String platform,
    required OnLogEvent onLogEvent,
  }) async {
    this._pureBasicUrl = pureBasicUrl;
    this._onLogEvent = onLogEvent;
    _localPackage = (await PackageInfo.fromPlatform()).packageName;
    _payPackage = await _payResource.getPackageRes(packageUrl);
    _findPackageToBind();
  }

  void changeBehavior(String preferPackage, bool onlyPrefer) {
    this._preferPackage = preferPackage;
    _findPackageToBind(onlyPrefer);
  }

  void _findPackageToBind([bool onlyPrefer = false]) async {
    // 选择 prefer 偏好的 app
    _onlyChoosePrefer(onlyPrefer);

    // 如果 prefer 安装了，就选用 prefer
    await _applyChoosePrefer();

    // 如果 package 列表中有已经安装好了的，就选用此 app
    await _chooseInstalled();

    // 选择没有安装的 prefer
    _chooseUnInstall();

    // 选用列表中的第一个 app
    _chooseFirst();
  }

  void _chooseFirst() {
    try {
      _selectPackage ??= _payPackage.payPackageList?.firstWhere((package) {
          return package.packageName != _localPackage;
        });
    } catch (_) {}
  }

  void _chooseUnInstall() {
    try {
      _selectPackage ??= _payPackage.payPackageList?.firstWhere((package) {
          return package.packageName == _preferPackage && package.packageName != _localPackage;
        });
    } catch (_) {}
  }

  Future _chooseInstalled() async {
    if (_selectPackage == null && _payPackage.payPackageList != null ) {
      for (var package in _payPackage.payPackageList!) {
        var installed = await plugin.isAppInstalled(package.packageName);
        if (installed && _localPackage != package.packageName) {
          _selectPackage = package;
          break;
        }
      }
    }
  }

  Future _applyChoosePrefer() async {
    try {
      if (_selectPackage == null) {
        var isPreferPackageInstalled = await plugin.isAppInstalled(_preferPackage);
        _selectPackage = _payPackage.payPackageList?.firstWhere((package) {
          return package.packageName == _preferPackage &&
              package.packageName != _localPackage &&
              isPreferPackageInstalled;
        });
      }
    } catch (_) {}
  }

  void _onlyChoosePrefer(bool onlyPrefer) {
    try {
      if (onlyPrefer) {
        _selectPackage = _payPackage.payPackageList?.firstWhere((package) {
          return package.packageName == _preferPackage && package.packageName != _localPackage;
        });
      }
    } catch (_) {}
  }

  Future<PayResult> deliverResult(String price, int period, String periodType) async {
    var payResult = await buy(price, period, periodType);
    if (payResult.success) {
      payResult = await plugin.deliverResult();
    }

    if (_onLogEvent != null && int.parse(payResult.code) > -200) {
      _onLogEvent!(
        "superpay_receive_pay_response",
        {
          "result": payResult.success ? "200" : "400",
          "message": payResult.success
              ? "${jsonDecode(payResult.data["content"])}_${payResult.data["platform"]}"
              : payResult.errMessage?.split("---")[0],
          "platform": payResult.success ? payResult.data["platform"] : payResult.errMessage?.split(
              "---")[1],
        },
      );
    }
    return payResult;
  }

  Future<PayResult> buy(String price, int period, String periodType) async {
    if (_selectPackage == null) {
      return PayResult.failed(code: "-202", errMessage: "Not find.");
    }
    var isInstalled = await plugin.isAppInstalled(_selectPackage!.packageName);
    if (!isInstalled) {
      return PayResult.failed(code: "-203", errMessage: "Not installed.");
    }

    var params = {
      priceKey: price,
      periodKey: period,
      periodTypeKey: periodType,
      pureBasicUrlKey: _pureBasicUrl,
      packageNameKey: _selectPackage!.packageName,
      serviceNameKey: _selectPackage!.serviceName,
    };
    var start = await plugin.startActivityForResult(params);

    if (_onLogEvent != null) {
      _onLogEvent!(
        "superpay_send_pay_request",
        {
          "result": start ? "200" : "400",
          "data": {
            "price": price,
            "period": period,
            "periodType": periodType,
          },
        },
      );
    }
    return PayResult.success(null);
  }

  static const MethodChannel _channel = MethodChannel('fgx_super_pay');

  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }
}

class PayResult<T> {
  late T data;
  String code;
  late String? errMessage;
  bool success;

  PayResult({required this.data, required this.code, required this.errMessage, required this.success});

  PayResult.success(this.data)
      : success = true,
        code = "200";

  PayResult.failed({required this.code, required this.errMessage}) : success = false;
}